import streamlit as st
from streamlit_image_coordinates import streamlit_image_coordinates

from streamlit_extras import extra

streamlit_image_coordinates = extra(streamlit_image_coordinates)


def example():
    "# Click on the image"
    last_coordinates = streamlit_image_coordinates("https://placekitten.com/200/300")

    st.write(last_coordinates)


__title__ = "Streamlit Image Coordinates"
__desc__ = """
Allows you to add an image to your app, and get the coordinates of where the user last
clicked on the image."""
__icon__ = "🎯"
__examples__ = [example]
__author__ = "Zachary Blackwood"
__experimental_playground__ = False
__github_repo__ = "blackary/streamlit-image-coordinates"
__pypi_name__ = "streamlit-image-coordinates"
__package_name__ = "streamlit_image_coordinates"



import base64
from pathlib import Path

import streamlit as st
import validators

from .. import extra


@extra
def add_logo(logo_url: str, height: int = 120):
    """Add a logo (from logo_url) on the top of the navigation page of a multipage app.
    Taken from https://discuss.streamlit.io/t/put-logo-and-title-above-on-top-of-page-navigation-in-sidebar-of-multipage-app/28213/6

    The url can either be a url to the image, or a local path to the image.

    Args:
        logo_url (str): URL/local path of the logo
    """

    if validators.url(logo_url) is True:
        logo = f"url({logo_url})"
    else:
        logo = f"url(data:image/png;base64,{base64.b64encode(Path(logo_url).read_bytes()).decode()})"

    st.markdown(
        f"""
        <style>
            [data-testid="stSidebarNav"] {{
                background-image: {logo};
                background-repeat: no-repeat;
                padding-top: {height - 40}px;
                background-position: 20px 20px;
            }}
        </style>
        """,
        unsafe_allow_html=True,
    )


def example():
    if st.checkbox("Use url", value=True):
        add_logo("http://placekitten.com/120/120")
    else:
        add_logo("gallery/kitty.jpeg", height=300)
    st.write("👈 Check out the cat in the nav-bar!")


__title__ = "App logo"
__desc__ = "Add a logo on top of the navigation bar of a multipage app"
__icon__ = "🐱"
__examples__ = [example]
__author__ = "Zachary Blackwood"
__experimental_playground__ = True



import streamlit as st

from streamlit_extras import extra


@extra
def style_metric_cards(
    background_color: str = "#FFF",
    border_size_px: int = 1,
    border_color: str = "#CCC",
    border_radius_px: int = 5,
    border_left_color: str = "#9AD8E1",
    box_shadow: bool = True,
):

    box_shadow_str = (
        "box-shadow: 0 0.15rem 1.75rem 0 rgba(58, 59, 69, 0.15) !important;"
        if box_shadow
        else "box-shadow: none !important;"
    )
    st.markdown(
        f"""
        <style>
            div[data-testid="metric-container"] {{
                background-color: {background_color};
                border: {border_size_px}px solid {border_color};
                padding: 5% 5% 5% 10%;
                border-radius: {border_radius_px}px;
                border-left: 0.5rem solid {border_left_color} !important;
                {box_shadow_str}
            }}
        </style>
        """,
        unsafe_allow_html=True,
    )


def example():
    col1, col2, col3 = st.columns(3)

    col1.metric(label="Gain", value=5000, delta=1000)
    col2.metric(label="Loss", value=5000, delta=-1000)
    col3.metric(label="No Change", value=5000, delta=0)

    style_metric_cards()


def _add_metric_card_cards():
    """For playground"""
    col1, col2, col3 = st.columns(3)

    col1.metric(label="Gain", value=5000, delta=1000)
    col2.metric(label="Loss", value=5000, delta=-1000)
    col3.metric(label="No Change", value=5000, delta=0)


__title__ = "Metric Cards"
__desc__ = "Restyle metrics as cards"
__icon__ = "♠️"
__examples__ = [example]
__author__ = "Chanin Nantasenamat"
__experimental_playground__ = True
__experimental_playground_funcs__ = [_add_metric_card_cards]



import streamlit as st
from st_keyup import st_keyup

from .. import extra

st_keyup = extra(st_keyup)


def example():
    st.write("## Notice how the output doesn't update until you hit enter")
    out = st.text_input("Normal text input")
    st.write(out)
    st.write("## Notice how the output updates with every key you press")
    out2 = st_keyup("Keyup input")
    st.write(out2)


def example_with_debounce():
    st.write("## Notice how the output doesn't update until 500ms has passed")
    out = st_keyup("Keyup with debounce", debounce=500)
    st.write(out)


__title__ = "Keyup text input"
__desc__ = "A text input that updates with every key press"
__icon__ = "🔑"
__author__ = "Zachary Blackwood"
__examples__ = [example, example_with_debounce]
__github_repo__ = "blackary/streamlit-keyup"
__pypi_name__ = "streamlit-keyup"
__package_name__ = "st_keyup"
__experimental_playground__ = False



from contextlib import contextmanager
from typing import Generator, Sequence

import pandas as pd
import streamlit as st

try:
    from streamlit import cache_data  # streamlit >= 1.18.0
except ImportError:
    from streamlit import experimental_memo as cache_data  # streamlit >= 0.89

from .. import extra


@cache_data
def _to_csv(data: pd.DataFrame):
    return data.to_csv().encode("utf-8")


@cache_data
def _to_parquet(data: pd.DataFrame):
    return data.to_parquet()


_SUPPORTED_EXPORTS = {
    "CSV": {
        "function": _to_csv,
        "extension": ".csv",
        "mime": "text/csv",
    },
    "Parquet": {
        "function": _to_parquet,
        "extension": ".parquet",
    },
}

_SUPPORTED_EXPORT_KEYS = list(_SUPPORTED_EXPORTS.keys())


@extra  # type: ignore
@contextmanager
def chart_container(
    data: pd.DataFrame,
    tabs: Sequence[str] = (
        "Chart 📈",
        "Dataframe 📄",
        "Export 📁",
    ),
    export_formats: Sequence[str] = _SUPPORTED_EXPORT_KEYS,
) -> Generator:
    """Embed chart in a (chart, data, export, explore) tabs container to let the viewer explore and export its underlying data.

    Args:
        data (pd.DataFrame): Dataframe used in the dataframe tab.
        tabs (Sequence, optional): Tab labels. Defaults to ("Chart 📈", "Dataframe 📄", "Export 📁").
        export_formats (Sequence, optional): Export file formats. Defaults to ("CSV", "Parquet")
    """

    assert all(
        export_format in _SUPPORTED_EXPORTS for export_format in export_formats
    ), f"Input format is not supported, please use one within {_SUPPORTED_EXPORTS.keys()}"

    if "chart_container_widget_key" not in st.session_state:
        st.session_state["chart_container_widget_key"] = 0

    def _get_random_widget_key() -> str:
        st.session_state.chart_container_widget_key += 1
        return st.session_state.chart_container_widget_key

    tab_1, tab_2, tab_3 = st.tabs(tabs)

    with tab_1:
        yield

    with tab_2:
        st.dataframe(data, use_container_width=True)

    with tab_3:
        st.caption("Export limited to 1 million rows.")
        export_data = data.head(1_000_000)
        for chosen_export_format in export_formats:
            export_utils = _SUPPORTED_EXPORTS[chosen_export_format]
            exporter = export_utils["function"]
            extension = export_utils["extension"]
            st.download_button(
                f"Download data as {extension}",
                data=exporter(export_data),
                file_name="data" + extension,
                mime=export_utils.get("mime"),
                key=_get_random_widget_key(),
            )


def _get_random_data():
    return pd.DataFrame(
        {
            "a": {
                "0": -0.0303305049,
                "1": -0.1351647028,
                "2": 1.5870779911,
                "3": -1.3836484569,
                "4": 0.098720535,
                "5": -0.3647604289,
                "6": 0.9750224677,
                "7": 1.9491871558,
                "8": 1.2551013828,
                "9": 0.7843967348,
                "10": -2.1718875405,
                "11": -0.5735476161,
                "12": -0.1762156189,
                "13": -0.7407221386,
                "14": 0.0250880712,
                "15": 0.2507830305,
                "16": -1.6129273914,
                "17": -0.4366293705,
                "18": 0.6685293897,
                "19": -0.6817020803,
            },
            "b": {
                "0": -1.0967545261,
                "1": -1.7796399982,
                "2": -0.3882711116,
                "3": -0.9394858767,
                "4": -1.2070381231,
                "5": 0.1399939192,
                "6": -0.2630170362,
                "7": -1.860974035,
                "8": -0.7970238226,
                "9": -0.2423244595,
                "10": -1.5030435135,
                "11": -0.5431451721,
                "12": 0.7148690473,
                "13": 0.2743864135,
                "14": -0.4985624255,
                "15": 0.8893880137,
                "16": -0.1999725001,
                "17": 0.7686599597,
                "18": -0.309580067,
                "19": 1.4339088721,
            },
            "c": {
                "0": -1.5651083451,
                "1": 0.2913816082,
                "2": 1.1995339283,
                "3": -1.4542600362,
                "4": -0.3883294163,
                "5": 0.7431884696,
                "6": -0.7326326269,
                "7": -0.6609734474,
                "8": 0.8558914313,
                "9": 1.8783811641,
                "10": -1.6879024826,
                "11": -1.3691829023,
                "12": 1.0217629925,
                "13": 1.0185489658,
                "14": 0.2889080313,
                "15": -0.0821493491,
                "16": 0.3449393609,
                "17": 0.2998609069,
                "18": -0.3677563609,
                "19": 0.4388375331,
            },
        }
    )


def example_one():
    chart_data = _get_random_data()
    with chart_container(chart_data):
        st.write("Here's a cool chart")
        st.area_chart(chart_data)


def example_two():
    chart_data = _get_random_data()
    with chart_container(chart_data):
        st.write(
            "I can use a subset of the data for my chart... "
            "but still give all the necessary context in "
            "`chart_container`!"
        )
        st.area_chart(chart_data[["a", "b"]])


__title__ = "Chart container"
__desc__ = "Embed your chart in a nice tabs container to let viewers explore and export its underlying data."
__icon__ = "🖼️"
__examples__ = [
    example_one,
    example_two,
]
__author__ = "Arnaud Miribel"



from __future__ import annotations

from typing import Any, Iterable

import pandas as pd
import streamlit as st

from streamlit_extras import extra


def _transform_arguments(*args, **kwargs) -> tuple[str, Iterable[Any], dict[str, Any]]:
    no_selection_label = kwargs.pop("no_selection_label", "---")

    _args = list(args)

    # Get the options from either the args or kwargs
    try:
        options = _args.pop(1)
    except IndexError:
        options = kwargs["options"]

    # Prepend the no-selection option to the list of options
    if no_selection_label not in options:
        if isinstance(options, pd.Series):
            options = list(pd.concat([pd.Series([no_selection_label]), options]))
        elif isinstance(options, pd.DataFrame):
            # If the options are a DataFrame, the options are just the first column
            options = options.iloc[:, 0]
            options = list(pd.concat([pd.Series([no_selection_label]), options]))
        else:
            options = [no_selection_label] + list(options)
        kwargs["options"] = options

    return no_selection_label, _args, kwargs


@extra
def selectbox(*args, **kwargs):
    """A selectbox that returns None unless the user has explicitly selected one of the
    options.

    All arguments are passed to st.selectbox except for `no_selection_label`, which is
    used to specify the label of the option that represents no selection.

    Parameters
    ----------
    no_selection_label : str
        The label to use for the no-selection option. Defaults to "---".
    """
    no_selection_label, _args, _kwargs = _transform_arguments(*args, **kwargs)

    result = st.selectbox(*_args, **_kwargs)
    if result == no_selection_label:
        return None
    return result


def example():
    st.write(
        """
        This is an example of a selectbox that returns None unless the user has
        explicitly selected one of the options.

        The selectbox below has no default value, so it will return None until the
        user selects an option.
        """
    )
    result = selectbox("Select an option", ["A", "B", "C"])
    st.write("Result:", result)

    result = selectbox(
        "Select an option with different label",
        ["A", "B", "C"],
        no_selection_label="<None>",
    )
    st.write("Result:", result)


__title__ = "No-Default Selectbox"
__desc__ = """
Just like st.selectbox, but with no default value -- returns None if nothing is selected.

Meant to be a solution to https://github.com/streamlit/streamlit/issues/949
"""
__icon__ = "🗳️"
__examples__ = [example]
__author__ = "Zachary Blackwood"
__experimental_playground__ = False



import time

import streamlit as st

from .. import extra

center_css = """
<style>

div[class*="StatusWidget"]{

    position: fixed;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    width: 50%;
}

</style>
"""


@extra
def center_running():
    st.markdown(center_css, unsafe_allow_html=True)


def example():
    click = st.button("Observe where the 🏃‍♂️ running widget is now!")
    if click:
        center_running()
        time.sleep(2)


__title__ = "Customize running"
__desc__ = "Customize the running widget"
__icon__ = "🏃‍♂️"
__examples__ = [example]
__author__ = "koninhoo"
__forum_url__ = (
    "https://discuss.streamlit.io/t/change-the-running-widget-position/30466"
)
__experimental_playground__ = False



from streamlit_embedcode import (
    codepen_snippet,
    github_gist,
    gitlab_snippet,
    ideone_snippet,
    pastebin_snippet,
    tagmycode_snippet,
)

from .. import extra

codepen_snippet = extra(codepen_snippet)
github_gist = extra(github_gist)
gitlab_snippet = extra(gitlab_snippet)
ideone_snippet = extra(ideone_snippet)
pastebin_snippet = extra(pastebin_snippet)
tagmycode_snippet = extra(tagmycode_snippet)


def example_github():
    github_gist(
        "https://gist.github.com/randyzwitch/be8c5e9fb5b8e7b046afebcac12e5087/",
        width=700,
        height=400,
    )


def example_gitlab():
    gitlab_snippet(
        "https://gitlab.com/snippets/1995463",
        width=700,
        height=200,
    )


def example_codepen(codepen_snippet):
    codepen_snippet(
        "https://codepen.io/randyzwitch/pen/GRrYrBw",
        width=700,
        height=400,
    )


def example_ideone(ideone_snippet):
    ideone_snippet(
        "https://ideone.com/5V7XZ6",
        width=700,
        height=400,
    )


def example_pastebin(pastebin_snippet):
    pastebin_snippet(
        "https://pastebin.com/8QZ7YjYD",
        width=700,
        height=400,
    )


def example_tagmycode(tagmycode_snippet):
    tagmycode_snippet(
        "https://tagmycode.com/snippet/1038",
        width=700,
        height=400,
    )


__title__ = "Embed code"
__desc__ = "Embed code from various platforms (Gists, snippets...)"
__icon__ = "📋"
__examples__ = {
    example_github: [github_gist],
    example_gitlab: [gitlab_snippet],
}
__author__ = "randyzwitch"
__github_repo__ = "randyzwitch/streamlit-embedcode"
__pypi_name__ = "streamlit-embedcode"
__package_name__ = "streamlit_embedcode"
__experimental_playground__ = False
